import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';

abstract class AccessEvent extends Equatable {
  const AccessEvent();

  @override
  List<Object> get props => [];
}

class FetchTableData extends AccessEvent {}

class ResetSearch extends AccessEvent {}

class TabChangedEvent extends AccessEvent {
  final int selectedIndex;

  const TabChangedEvent(this.selectedIndex);
}

class SelectTime extends AccessEvent {
  final BuildContext context;
  final bool isStart;
  const SelectTime({required this.context, required this.isStart});
  @override
  List<Object> get props => [context, isStart];
}

class FilterDataEvent extends AccessEvent {
  final String? passwordName;
  final String? emailAuthorizer;
  final int? startTime;
  final int? endTime;
  final int selectedTabIndex; // Add this field

  const FilterDataEvent({
    this.passwordName,
    this.emailAuthorizer,
    this.startTime,
    this.endTime,
    required this.selectedTabIndex, // Initialize this field
  });
}
